<?php
namespace nur\sery\db;

/**
 * Interface ICapacitor: objet permettant d'accumuler des données pour les
 * réutiliser plus tard
 */
interface ICapacitor {
  /** tester si le canal spécifié existe */
  function exists(?string $channel=null): bool;

  /** supprimer le canal spécifié */
  function reset(?string $channel=null): void;

  /** charger une valeur dans le canal */
  function charge($item, ?string $channel=null): void;

  /** décharger les données du canal spécifié */
  function discharge($keys=null, ?string $channel=null, ?bool $reset=null): iterable;

  /**
   * obtenir l'élément identifié par les clés spécifiées sur le canal spécifié
   *
   * si $keys n'est pas un tableau, il est transformé en ["_id" => $keys]
   */
  function get($keys, ?string $channel=null);

  /**
   * appeler une fonction pour chaque élément du canal spécifié.
   *
   * $keys permet de filtrer parmi les élements chargés
   *
   * si $func retourne un tableau, il est utilisé pour mettre à jour
   * l'enregistrement.
   */
  function each($keys, callable $func, ?array $args=null, ?string $channel=null): void;

  function close(): void;
}
